/***************************************************************************//**
 *   @file   AD5449.c
 *   @brief  Implementation of AD5449 Driver.
 *   @author Dan Nechita
********************************************************************************
 * Copyright 2012(c) Analog Devices, Inc.
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *  - Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *  - Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *  - Neither the name of Analog Devices, Inc. nor the names of its
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *  - The use of this software may or may not infringe the patent rights
 *    of one or more patent holders.  This license does not release you
 *    from the requirement that you obtain separate licenses from these
 *    patent holders to use this software.
 *  - Use of the software either in source or binary form, must be run
 *    on or directly connected to an Analog Devices Inc. component.
 *
 * THIS SOFTWARE IS PROVIDED BY ANALOG DEVICES "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, NON-INFRINGEMENT,
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL ANALOG DEVICES BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, INTELLECTUAL PROPERTY RIGHTS, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
********************************************************************************
 *   SVN Revision: 584
*******************************************************************************/

/******************************************************************************/
/* Include Files                                                              */
/******************************************************************************/
#include "AD5449.h"		// AD5449 definitions.



/***************************************************************************//**
 * @brief Initializes communication. 
 *
 * @return 1.
*******************************************************************************/

unsigned char AD5449_Init()
{
    SPI_Init(0, 1000000, 1, 0);
    AD5449_LDAC_OUT;
    AD5449_CLR_OUT;
    AD5449_LDAC_LOW; // Active low.
    AD5449_CLR_HIGH; // Active low.
    
    return(1); 
}

/***************************************************************************//**
 * @brief  Sends a 16-bit value to the Input Shift Register of the device. 
 *
 * @param command - The 4 control bits of the DAC.
 *                  Example: AD5449_CTRL_NOP - No operation (power-on default).
 *                           AD5449_CTRL_A_LOAD_UPDATE -  Load and update.
 *                           AD5449_CTRL_DISBL_DAISY - Disable daisy-chain.
 * @param dataWord - 12-bit of data.
 *
 * @return none.
*******************************************************************************/
void AD5449_SetRegisterValue(unsigned short command, unsigned short dataWord)
{
    unsigned char data[3] = {0, 0, 0};
    unsigned long spiWord = 0;
    unsigned char *pspiWord = (unsigned char*)&spiWord;
    
    spiWord = AD5449_CTRL_BITS(command) | AD5449_DATA_BITS(dataWord);
    data[0] = 0x01;
    data[1] = pspiWord[1];
    data[2] = pspiWord[0];
    SPI_Write(data, 2);
}


/***************************************************************************//**
 * @brief  Reads 12-bit data form DAC register A or DAC register B.
 *
 * @param dacChannel - Reads form Channel A or B.
 *                     Example: 0 - Channel A
 *                              1 - Channel B
 *
 * @return spiRead - 12-bit value .
*******************************************************************************/
unsigned short AD5449_GetRegisterValue(unsigned char dacChannel)
{
    unsigned char data[4] = {0, 0, 0};
    unsigned long spiWord = 0;
    unsigned char *pspiWord = (unsigned char*)&spiWord;
    unsigned short spiRead = 0;
    
    if(dacChannel == 0)
    {
        spiWord = AD5449_CTRL_BITS(AD5449_CTRL_A_READBACK);
    }
    else if(dacChannel == 1)
    {
        spiWord = AD5449_CTRL_BITS(AD5449_CTRL_B_READBACK);
    }
    data[0] = 0x01;
    data[1] = pspiWord[1];
    data[2] = pspiWord[0];
    SPI_Write(data, 2);
    data[0] = 0x01;
    data[1] = 0x00;
    data[2] = 0x00;
    SPI_Read(data, 2);
    spiRead = (data[0] << 8) + data[1]; 
    
    return(spiRead);
}

/***************************************************************************//**
 * @brief  Enables specific options available for the device.
 *
 * @param setupWord - A value loaded to Control Register.
 *                    Example:         
 *                             AD5449_SDO1 - Weak SDO driver.
 *                             AD5449_SDO2 - SDO configured as open drain.
 *                             AD5449_DSY  - Enables daisy-chain mode.
 *                             AD5449_HCLR - Clears DAC output to midscale.
 *                             AD5449_SCLK - Clock on rising edge.
 *
 * @return none.
*******************************************************************************/
void AD5449_Setup(unsigned short setupWord)
{
  unsigned char data[4] = {0, 0, 0};
  unsigned char *psetupWord = (unsigned char*)&setupWord;   
  
  setupWord = AD5449_CTRL_BITS(AD5449_CTRL_CONTROL_WORD) |
              setupWord;
  data[0] = 0x01;
  data[1] = psetupWord[1];
  data[2] = psetupWord[0];
  SPI_Write(data, 2);
}
